import javax.swing.UIManager;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.lang.*;
import java.util.*;
import java.io.*;

import gl4java.*;
import gl4java.GLContext;
import gl4java.awt.GLAnimCanvas;
import gl4java.awt.GLCanvas;
import gl4java.utils.glut.*;
import gl4java.utils.textures.*;

/**
* En klasse som tegner en 3-dimenasjonal blekksprut ved hjelp av gl4java.<br>
* Bruker objekter av klassene Head, Eye, CVector3 og Material.
* @author Linda Cecilie Kjeldsen, Grafisk Databehandling 2004, Hi
* @version 0.1
*/

public class Octopus extends JFrame implements ActionListener {
	JPanel contentPane;
	
	WindowHandler wHandler;
	MouseHandler mHandler;
	
	// Variabler for musehndtering
	float rotx = 0.0f;
	float rotz = 0.0f;
	float lastx = 0.0f;
	float lastz = 0.0f;
	float delta_rotx = 1.0f;
	float delta_rotz = 1.0f;
	float movey = 0;
	float movez = 0;
	boolean isMoving = false;
	
	// Variabler for menyen
	MenuBar menybar;
	Menu meny;
	MenuItem swim;
	MenuItem back;
	MenuItem exit;
	
	// gl4java canvas
	myGLCanvas canvas = null;
	
	/**
	* Konstruktr for Octopus-framen<br>
	* Setter opp menylinje og GLAnimCanvas til  tegne p
	*/ 
	public Octopus() {
		contentPane = (JPanel) this.getContentPane();
		contentPane.setLayout(new BorderLayout());
		this.setSize(new Dimension(750, 600));
		this.setTitle("...Octopus...");
		
		// Sett opp menyen
	    menybar = new MenuBar();
	    this.setMenuBar(menybar);
	    meny = new Menu("Blekksprut");
	     
	    menybar.add(meny);
	         
	    swim = new MenuItem("Svmmetur");
	    meny.add(swim);
		swim.addActionListener(this);
		 
		back = new MenuItem("Utgangsposisjon");
	    meny.add(back);
		back.addActionListener(this);
		
	    meny.addSeparator();
	
	    exit = new MenuItem("Exit");
	    meny.add(exit);
	    exit.addActionListener(this);
		
		// Sett opp GLcanvasen
		Dimension d = getSize();
		canvas = new myGLCanvas(d.width, d.height);
		contentPane.add("Center", canvas);
		
		mHandler = new MouseHandler();
		canvas.addMouseListener(mHandler);
		canvas.addMouseMotionListener(mHandler);
		
		// Sett opp en vindushndterer
		wHandler=new WindowHandler();
		this.addWindowListener(wHandler);
		
		pack();
		show();
	}
	
	/**
	* Main metode
	*/
	public static void main(String[] args) {
		new Octopus();
	}
	
	/**
	* Metode for hndtering av brukers handlinger
	* @param e den eventen som skal utfres
	*/  
	public void actionPerformed(ActionEvent e) {
		if(e.getSource() == swim) {
			// Send blekkspruten p svmmetur
			canvas.isSwimming = true;
				canvas.setSwimPoints();
				canvas.start();
		}
		if(e.getSource() == back) {
			// Slutt p svmmeturen
			canvas.stop();
			canvas.isSwimming = false;
			canvas.resetSwimPoints();
		}
		if(e.getSource() == exit) {
			System.exit(0);
		}
	}

// --------------------------------------------
	/**
	* Intern klasse for vindushndtering
	* @see WindowListener
	*/
  	class WindowHandler implements WindowListener {
		public void windowOpened(WindowEvent e) {}
		public void windowClosing(WindowEvent e) {
			System.exit( 0 );
		}
		public void windowClosed(WindowEvent e) {}
		public void windowIconified(WindowEvent e) {}
		public void windowDeiconified(WindowEvent e) {}
		public void windowActivated(WindowEvent e) {}
		public void windowDeactivated(WindowEvent e) {}
	}

// --------------------------------------------
	/**
	* Intern klasse for musehndtering
	* @see MouseListener
	* @see MouseMotionListener
	*/
  	class MouseHandler implements MouseListener, MouseMotionListener {
	  	public void mouseEntered(MouseEvent e) {}
	  	public void mouseExited(MouseEvent e) {}
	  	public void mouseMoved(MouseEvent e) {}
	  	public void mouseClicked(MouseEvent e) {}
	  	public void mouseReleased(MouseEvent e) {
		  	isMoving = false;
		  	canvas.repaint();
		}
		public void mousePressed(MouseEvent e) {
			lastx = e.getX();
			lastz = e.getY();
			isMoving = true;
		}
		public void mouseDragged(MouseEvent e) {
			if(!isMoving) return;
			
			int x = e.getX();
			int y = e.getY();
			
			Dimension dim = e.getComponent().getSize();
			int width = dim.width;
			int height = dim.height;
			
			rotz += (90.0f * ((float)(x-lastx)/(float)(width)));
			rotx += (90.0f * ((float)(y-lastz)/(float)(height)));
			
			lastx = x;
			lastz = y;
			
			canvas.repaint();
		}
	}

//-------------------------------------------------------
	/**
	* Intern klasse som tar seg av tegningen av blekkspruten
	*/
	private class myGLCanvas extends GLAnimCanvas {
		
		// Objekter som holder kontrollpunkter for blekksprutens hode og yne
		Head head = new Head();
		Eye eye = new Eye();
		
		// Objekt som setter opp materiale som brukes dersom teksturene ikke kan lastes
		Material material = new Material(gl);
		
		// Variabler for animasjon
		boolean isSwimming = false;
		int frameCounter = 0;
		
		// Array med koordinatene til teksturene, de legges en gang over hele flaten.  
		float textPoints[]={
		        0.0f, 0.0f,
		        1.0f, 0.0f,
		        0.0f, 1.0f,
		        1.0f, 1.1f,
		};
		
		int []texName = new int[1];
		
		GLUTFunc glut = null;
		
		// Variabler for  ta vare p armenes kontrollpunkter
		int MAX_STEPS = 80; // Antall punkter pr kurve (og sylindere pr arm)
		
		float btab[][] = new float[4][MAX_STEPS];	// ligningene for  regne ut bezierfunksjonen
		float d_btab[][] = new float[4][MAX_STEPS]; // ligningene for  regne ut den deriverte
		
		float bez[][] = new float[MAX_STEPS][3]; 	// verdiene til bezierfunksjonen
		float d_bez[][] = new float[MAX_STEPS][3]; 	// verdiene til den deriverte
		
		CVector3 g_vStartPoint;
		CVector3 g_vControlPoint1;
		CVector3 g_vControlPoint2;
		CVector3 g_vEndPoint;
		
		// Punkter for svmming
		float ssx = -0.25f, ssy = -1.75f, ssz = 0;
		float c1sx = 0, c1sy = -1, c1sz = 0;
		float c2sx = -0.5f, c2sy = -0.25f, c2sz = 0;
		float esx = 0, esy = 0.25f, esz = 0;
		
		// Punkter for  bestemme strrelsen p arm-sylinderene
		float cylRad = 0.0225f;
		float incFac = 1.0225f;
		
		/**
		* Konstruktr for myGLCanvas
		* @param w bredden p komponenten
		* @param h hyden p komponenten
		*/
		public myGLCanvas(int w, int h) {
			super(w, h);
			
			// Sett opp matrisene som holder ligningene for  regne ut kurven og den deriverte
			int tix=0;
			while(tix<MAX_STEPS) {
				float t=(1.0f/(1.0f*MAX_STEPS))*(1.0f*tix);
				btab[0][tix]=(1-t)*(1-t)*(1-t);
				btab[1][tix]=3.0f*t*(1-t)*(1-t);
				btab[2][tix]=3.0f*t*t*(1-t);
				btab[3][tix]=t*t*t;
				tix++;
			}
			tix=0;
			while(tix<MAX_STEPS) {
				float t=(1.0f/(1.0f*MAX_STEPS))*(1.0f*tix);
				d_btab[0][tix]=-3.0f*t*t+6.0f*t-3.0f;
				d_btab[1][tix]=9.0f*t*t-12.0f*t+3.0f;
				d_btab[2][tix]=-9.0f*t*t+6.0f*t;
				d_btab[3][tix]=3.0f*t*t;
				tix++;
			}
		}
		
		/**
	     * Initialiserer canvasen fr GLContext er opprettet.
	     * <p>
	     * Blir kalt like FR GLContext blir opprettet.
	     * <p>
	     * Definerer et fornuftig sett med 3D options til denne komponenten.  
	     */
		public void preInit() {
			doubleBuffer = true;
			stereoView = false;
			accumSize = 8;
		}
		
		/**
	     * Initialiserer canvasen etter at GLContext er opprettet.
	     * <p>
	     * Blir kalt like ETTER GLContext er opprettet. Setter opp OpenGLs tilstandsmatrise.
	     * Setter opp lyset, glatter modellen, aktiviserer dybdetesting, og setter bakgrunnsfarge.
	     */
		public void init() {
			glut = new GLUTFuncLightImpl(gl, glu);
			
			reshape(getSize().width, getSize().height);
			
			// Setter opp lyset
			float ambient[] = {0.4f,0.4f,0.4f,1.0f };
			float diffuse[] = {1.0f,1.0f,1.0f,1.0f };
			float position[] = {1.0f,1.0f,1.0f,0.0f };
			float lmodel_ambient[] = {0.4f,0.4f,0.4f,1.0f };
			
			gl.glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
			gl.glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
			gl.glLightfv(GL_LIGHT0, GL_POSITION, position);
			
			gl.glEnable(GL_LIGHT0);
			gl.glEnable(GL_LIGHTING);
			
			gl.glShadeModel(GL_SMOOTH);
			
			gl.glEnable(GL_NORMALIZE);
			
			gl.glEnable(GL_DEPTH_TEST);
			gl.glDepthFunc(GL_LESS);
			
			gl.glClearColor(0.9f, 0.9f, 0.9f, 1); // Setter bakgrunnen til lysegr
		}
		
		/**
		* Forsikrer at GL er korrekt initialisert.
		* Kaller showScene() hvis GL4Java er initialisert, og gjr
		* GLContexten current for denne trden
		*/
		public void display() {
			if (glj.gljMakeCurrent() == false)
				return;
						
			// Gjr action her, GLAnimCanvas kjrer selv...
			if(isSwimming) {
        		frameCounter++;
        		swim();
			}
				
			showScene(true);
			
			glj.gljSwap();
			glj.gljFree();
		}
		
		/**
		* Klarerer de ndvendige bufferene, setter opp transformasjonsmatrisen for viewet,
		* og kaller metodene som tegner de forskjellige delene av blekkspruten.
		*/
		public void showScene(boolean bRender) {
			gl.glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			gl.glClear(GL_ACCUM_BUFFER_BIT);
			gl.glLoadIdentity();
			
			// Setter opp view-matrisen
			glu.gluLookAt(0.0f, 0.0f, 5.0f, // yets posisjon
						0.0f, 0.0f, 0.0f, 	// Hva ser vi p
						0.0f, 1.0f, 0.0f);	// Hva er opp
			
			gl.glRotatef(rotx, 1.0f, 0.0f, 0.0f);
			gl.glRotatef(rotz, 0.0f, 0.0f, 1.0f);
			gl.glTranslatef(0,movey,movez); // Flytter blekkspruten nr den svmmer, ellers er movey og movez 0.
			
			gl.glPushMatrix();
			
			gl.glEnable(GL_TEXTURE_2D); 	//Skal bruke teksturer, s vi enabler de     
			drawBodyPart("head");
			gl.glDisable(GL_TEXTURE_2D); 	//Disabler teksturene igjen
			
			//drawBulbCtrPointLines();
			
			gl.glEnable(GL_TEXTURE_2D);      
			drawBodyPart("left");
			gl.glDisable(GL_TEXTURE_2D);
			
			gl.glEnable(GL_TEXTURE_2D);      
			drawBodyPart("right");
			gl.glDisable(GL_TEXTURE_2D);
			
			gl.glEnable(GL_TEXTURE_2D);
			drawArms();
			gl.glDisable(GL_TEXTURE_2D);
			
			// Tegne aksene
			/*
			gl.glDisable(GL_LIGHTING);
			gl.glLineWidth(2);
			gl.glBegin(GL_LINES);
				// rd x-akse
				gl.glColor3f(1,0,0); 
				gl.glVertex3f(0.5f,0,0);
				gl.glVertex3f(0,0,0);
				// grnn y-akse
				gl.glColor3f(0,1,0);
				gl.glVertex3f(0,1,0);
				gl.glVertex3f(0,0,0);
				// bl z-akse
				gl.glColor3f(0,0,1);
				gl.glVertex3f(0,0,1);
				gl.glVertex3f(0,0,-1);
			gl.glEnd();
			gl.glEnable(GL_LIGHTING);
			*/
							
			gl.glPopMatrix();
			gl.glFlush();
		}
		
		/**
		* Kalles nr strrelsen p canvasen endres
		* @param w bredden p komponenten
		* @param h hyden p komponenten
		*/
		public void reshape(int w, int h) {
			gl.glViewport(0, 0, w, h);
			gl.glMatrixMode(GL_PROJECTION);
			gl.glLoadIdentity();
			glu.gluPerspective(45.0f,(float)w/(float)h, 0.5 ,150.0f);
			gl.glMatrixMode(GL_MODELVIEW);
		}
		
		/** 
		* Setter opp kontrollpunkter for armene
		* @param index Forteller hvilke kontrollpunkter som skal beskrive kurven
		*/
		private void setupArm(int index) {
			// Alle armene m ende p ca samme sted (p midten under blekkspruten)
			g_vControlPoint2 = new CVector3(-1, 0.125f, 0);	// Det andre kontrollpunktet
			g_vEndPoint = new CVector3(0, 0, 0);			// Bezierkurvens endepunkt
			
			// Armenes posisjon nr programmet starter:
			if(index==0) {
				g_vStartPoint = new CVector3(-2, 0, 0);				// Bezierkurvens startpunkt
				g_vControlPoint1 = new CVector3(-1.5f, -0.5f, 0);	// Det frste kontrollpunktet
			} else if(index==1) {
				g_vStartPoint = new CVector3(-2, 0, 0);
				g_vControlPoint1 = new CVector3(-1.5f, 0.5f, 0);
			} else if(index==2) {
				g_vStartPoint = new CVector3(-1.75f, 0, 0);
				g_vControlPoint1 = new CVector3(-1, 0, 0);
			} else if(index==3) {
				g_vStartPoint = new CVector3(-2, 0, 0);
				g_vControlPoint1 = new CVector3(-1.5f, -0.25f, 0);
			} else if(index==4) {
				g_vStartPoint = new CVector3(-2, 0, 0);
				g_vControlPoint1 = new CVector3(-1.5f, -0.75f, 0);
			} else if(index==5) {
				g_vStartPoint = new CVector3(-2, 0, 0);
				g_vControlPoint1 = new CVector3(-1.5f, -0.15f, 0);
			} else if(index==6) {
				g_vStartPoint = new CVector3(-2.25f, 0, 0);
				g_vControlPoint1 = new CVector3(-1.5f, 0f, 0);
			} else if(index==7) {
				g_vStartPoint = new CVector3(-1.5f, 0, 0);
				g_vControlPoint1 = new CVector3(-1.5f, -1, 0);
			}
			
			// Setter opp matrisene som holder kurven og den deriverte
			makeBezier(g_vStartPoint, g_vControlPoint1, g_vControlPoint2, g_vEndPoint);
			make_d_Bezier(g_vStartPoint, g_vControlPoint1, g_vControlPoint2, g_vEndPoint);
		}
		
		/**
		* Kaller de metodene som gjr de ndvendige beregningene for  tegne blekksprutens armer
		*/
		public void drawArms() {
			textures("skin64_tex.png");
			material.purple();
			setTexParameteri();
			gl.glPushMatrix();
				
			for(int i = 0; i < 8; i++) { 		// Lkke for  f 8 armer
				if(!isSwimming) setupArm(i); 	// Setter opp nye punkter, slik at ikke alle armene skal ha samme kurve
				gl.glRotatef(45*i, 0, 1, 0); 	// Roterer 45 grader om origo fr hver arm
				
				int ix=0;
				cylRad = 0.0225f;
				
				while(ix < MAX_STEPS) {
					// Bruker den deriverte, som ligger langs kurven, som tangent
					CVector3 T = new CVector3(d_bez[ix][0],d_bez[ix][1],d_bez[ix][2]);
					T.normalize(); // Normaliserer T
			
					/* Finner en vektor ulik T, kan ikke bruke den dobbeltderiverte, fordi der kurven 
					   endrer retning, og aksene snur, mangler en sylinder og blekkspruten ser ut som 
					   den har t-skjorte p... */
					CVector3 B = new CVector3(d_bez[ix][0],d_bez[ix][1],d_bez[ix][2]); // B = T
					B.cross(0.0f,0.0f,1.0f); // B = T x (0.0, 0.0, 1.0)
					B.normalize(); // Normaliserer B
					
					CVector3 N = new CVector3(B.x,B.y,B.z); // N = B
					N.cross(T);  // N = B x T -> N er normalisert siden B og T er det
					
					// C er kurven selv
					CVector3 C = new CVector3(bez[ix][0],bez[ix][1],bez[ix][2]);
			
					float M[] = {
						N.x,N.y,N.z,0,
						B.x,B.y,B.z,0,
						T.x,T.y,T.z,0,
						C.x,C.y,C.z,1
					};
					
					// Tar vare p den matrisen som allerede eksisterer
					gl.glPushMatrix();
					
					// Multipliserer Frenet matrisen til OpenGLs matrise
					gl.glMultMatrixf(M);
			
					// Klar til  tegne
					long glCyl = glu.gluNewQuadric();
					
					if(ix == 0){
						// Tegner en liten kule i enden av hver arm fordi de er hule
						long glSphere = glu.gluNewQuadric();
						glu.gluQuadricTexture(glSphere,true);
						glu.gluSphere(glSphere,cylRad,20,20);
					}
					glu.gluQuadricTexture(glCyl,true);
					glu.gluCylinder(glCyl,cylRad,cylRad*incFac,(2/MAX_STEPS)+0.075,15,1);
					glu.gluDeleteQuadric(glCyl);
					gl.glPopMatrix();
					
					// neste t-posisjon
					ix++;
					
					// ker radius p den neste sylinderen 
					// 1.0225 er faktor for armene i utgangs- og flyteposisjon
					// 1.025 er faktor for armene i svmmeposisjon
					cylRad *= incFac;
				}
			}
			gl.glPopMatrix();
		}
		
		/** 
		* Hjelpefunksjon som tegner bezierfunksjonen "for hnd", 
		* alle punkter er kalkulert p forhnd
		*/ 
		private void drawBezier() {
			gl.glBegin(GL_LINE_STRIP);
			int ix = 0;
			while(ix < MAX_STEPS)
			{
				gl.glVertex3f(bez[ix][0],bez[ix][1],bez[ix][2]);
				ix++;
			}
			gl.glEnd();
		}
		
		/**
		* Kaller de metodene som gjr de ndvendige beregningene for  tegne blekksprutens hode eller yne
		* @param part Forteller hvilken kroppsdel som skal tegnes
		*/
		public void drawBodyPart(String part) {
			if (part.equals("head")) {
				textures("skin256_tex.png"); 	// Laster tekstur for hode
				material.purple();			// Legger p material i tilfelle teksturen ikke er tilgjengelig
			} else {
				textures("eye_tex.png"); 		// Laster tekstur for ye
				material.white();
				gl.glPushMatrix();			// Tar vare p matrisa, siden ynene roteres og flyttes p plass
				if (part.equals("left")) {
					// Roter og flytt, slik at det ikke er "glipper" mellom flatene
					gl.glRotatef(-67.5f,1.0f,0.0f,0.0f);
					gl.glRotatef(-22.5f,0.0f,0.0f,1.0f);
					gl.glTranslatef(-0.1f,-0.225f,0.6f);
				} else {
					// Roterer og flytter den andre veien for hyre ye   
					gl.glRotatef(-67.5f,1.0f,0.0f,0.0f);
					gl.glRotatef(22.5f,0.0f,0.0f,1.0f);
					gl.glTranslatef(0.1f,-0.225f,0.6f);
				}
				gl.glLightModeli(gl.GL_LIGHT_MODEL_TWO_SIDE,1);	
				gl.glEnable(gl.GL_MAP2_VERTEX_3);
				gl.glEnable(gl.GL_AUTO_NORMAL);
				gl.glEnable(gl.GL_NORMALIZE);
			}
			
			setTexParameteri();
			
			if (part.equals("head")) { 
				gl.glMap2f(GL_MAP2_VERTEX_3, 0.0f, 1.0f, 3, head.getUN(), 0.0f, 
							1.0f, 3*head.getUN(), head.getVN(), head.getCtrPoints());
			} else { 
				gl.glMap2f(GL_MAP2_VERTEX_3, 0.0f, 1.0f, 3,eye.getUN(), 0.0f,
							1.0f, 3*eye.getUN(), eye.getVN(), eye.getCtrPoints());
			}
			
			// Gjr klar til  legge ut teksturen        
			gl.glMap2f(GL_MAP2_TEXTURE_COORD_2,
			      0,1,2,2,
			      0,1,4,2,textPoints);  
			
			gl.glEnable(GL_MAP2_TEXTURE_COORD_2);
			
			// Aktiviserer Bezierflaten
			gl.glEnable(GL_MAP2_VERTEX_3);
			
			if (part.equals("head")) {
				// Stiller inn antall masker i "rutenettet"
				gl.glMapGrid2f(25, 0.0f, 1.0f, 25, 0.0f, 1.0f);
				
				// Roterer og tegner de to delene hodet bestr av
				gl.glPushMatrix(); 
				gl.glEvalMesh2(GL_FILL, 0, 25, 0, 25); // foran
				gl.glRotatef(180.0f, 0.0f, 1.0f, 0.0f);
				gl.glEvalMesh2(GL_FILL, 0, 25, 0, 25); // bak
				gl.glPopMatrix();
			} else {
				gl.glMapGrid2f(7, 0.0f, 1.0f, 7, 0.0f, 1.0f);
				
				// Tegner (hyre eller venstre) ye       
				gl.glEvalMesh2(gl.GL_FILL, 0, 7, 0, 7);
				gl.glPopMatrix();
			}	
		}
		
		/**
		* Hjelpefunksjon som tegner ut kontrollunktene til bezierflaten 
		* som utgjr blekksprutens hode
		*/
		public void drawBulbCtrPointLines() {
			gl.glLineWidth(1);
			// Svart farge til linjene
			float ambR[]={0.0f, 0.0f, 0.0f, 1.0f};
			float diffR[]={0.0f, 0.0f, 0.0f, 1.0f};
			float specR[]={0.0f, 0.0f, 0.0f, 1.0f};
			float shineR=32.00f;
			gl.glMaterialfv(GL_FRONT,GL_AMBIENT,ambR);
			gl.glMaterialfv(GL_FRONT,GL_DIFFUSE,diffR);
			gl.glMaterialfv(GL_FRONT,GL_SPECULAR,specR);
			gl.glMaterialf(GL_FRONT,GL_SHININESS,shineR);
			
			// Tellere
			int u,v;
			int step = 3 * head.getUN(); // 3-tallet betyr at kontrollpunktarrayene er tredimensjonale
			
			for(u=0;u<head.getUN();u++)
			{
				gl.glBegin(GL_LINE_STRIP);
				for(v=0;v<head.getVN();v++)
				{
					gl.glVertex3f(  head.getCtrPoints()[v*step+u*3+0],
						head.getCtrPoints()[v*step+u*3+1],
						head.getCtrPoints()[v*step+u*3+2]);
				}
				gl.glEnd();
			}
			
			for(v=0;v<head.getVN();v++)
			{
				gl.glBegin(GL_LINE_STRIP);	
				for(u=0;u<head.getUN();u++)
				{
					gl.glVertex3f(  head.getCtrPoints()[v*step+u*3+0],
						head.getCtrPoints()[v*step+u*3+1],
						head.getCtrPoints()[v*step+u*3+2]);
				}
				gl.glEnd();
			}
		}
		
		/**
		* Setter opp matrisen som holder punktene til bezierfunksjonen
		*/
		private void makeBezier(CVector3 p0, CVector3 p1, CVector3 p2, CVector3 p3) {
			int ix = 0;
			while(ix < MAX_STEPS) {
				bez[ix][0]=p0.x*btab[0][ix]+p1.x*btab[1][ix]+p2.x*btab[2][ix]+p3.x*btab[3][ix];
				bez[ix][1]=p0.y*btab[0][ix]+p1.y*btab[1][ix]+p2.y*btab[2][ix]+p3.y*btab[3][ix];
				bez[ix][2]=p0.z*btab[0][ix]+p1.z*btab[1][ix]+p2.z*btab[2][ix]+p3.z*btab[3][ix];
				ix++;
			}
		}
		
		/**
		* Setter opp matrisen som holder punktene til den deriverte
		*/
		private void make_d_Bezier(CVector3 p0, CVector3 p1, CVector3 p2, CVector3 p3) {
			int ix = 0;
			while(ix < MAX_STEPS) {
				d_bez[ix][0]=p0.x*d_btab[0][ix]+p1.x*d_btab[1][ix]+p2.x*d_btab[2][ix]+p3.x*d_btab[3][ix];
				d_bez[ix][1]=p0.y*d_btab[0][ix]+p1.y*d_btab[1][ix]+p2.y*d_btab[2][ix]+p3.y*d_btab[3][ix];
				d_bez[ix][2]=p0.z*d_btab[0][ix]+p1.z*d_btab[1][ix]+p2.z*d_btab[2][ix]+p3.z*d_btab[3][ix];
				ix++;
			}
		}
		
		/**
		* Returnerer variabelen isRunning
		* @return den boolske variabelen isRunning
		*/
		public boolean isItRunning() {
			return isRunning;
		}
		
		/**
		* Metode som laster teksturene
		*/
		public void textures(String file) {	 
			// Bruker .png bilder derfor PngTextureLoader  
			PngTextureLoader txtLoader = new PngTextureLoader(gl, glu);
		   	txtLoader.readTexture(file);
			if(txtLoader.isOk()) {
				// Genererer tekstur og legger den i arrayen   
				gl.glGenTextures(1,texName);  
				gl.glBindTexture(GL_TEXTURE_2D,texName[0]);
				// Her lages teksturen, gir hyde, bredde og annen informasjon om bildet
				gl.glTexImage2D(GL_TEXTURE_2D,0,3,
								txtLoader.getImageWidth(),
								txtLoader.getImageHeight(),
								0,GL_RGB,GL_UNSIGNED_BYTE,
								txtLoader.getTexture());
			}
	    }
	    
	    /**
	    * Hvordan teksturen skal legges ut
	    */
	    public void setTexParameteri() {
			gl.glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
			gl.glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
			gl.glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
			gl.glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
			gl.glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
		}
	    
	    /**
	    * Setter styreverdier til  passe armer i svmmeposisjon
	    */
	    public void setSwimPoints() {
		    cylRad = 0.025f;
		    incFac = 1.025f;
		    
		    g_vEndPoint = new CVector3(esx, esy, esz);
			g_vControlPoint2 = new CVector3(c2sx, c2sy, c2sz);
			g_vControlPoint1 = new CVector3(c1sx, c1sy, c1sz);
			g_vStartPoint = new CVector3(ssx, ssy, ssz);
			
			makeBezier(g_vStartPoint, g_vControlPoint1, g_vControlPoint2, g_vEndPoint);
			make_d_Bezier(g_vStartPoint, g_vControlPoint1, g_vControlPoint2, g_vEndPoint);
		}
		
		/**
		* Setter alle styreverdier tilbake til utgangsverdi
		*/
		public void resetSwimPoints() {
		    cylRad = 0.0225f;
		    incFac = 1.0225f;
			movey = 0;
			movez = 0;
		}
	    
		/**
		* Utfrer animasjonen
		*/
	    private void swim() {
		    /* Svmmepunkter: (armer samlet)
			End (0, 0.5f, 0), Control2 (-0.5f, -0.25f, 0), Control1 (0, -1, 0), Start (-0.25f, -1.75f, 0).	
			Flytepunkter: (armer utspent)
			End (0, 0, 0), Control2 (-1, 0.1f, 0), Control1 (-1.5f, -0.25f, 0), Start (-2, 0, 0).
			*/
			
			// Reset frameCounter for ny svmmebevegelse
			if(frameCounter > 10) frameCounter = 1;
			
			// Manipulerer alle kontrollpunktene til  nrme seg utspent armposisjon
			if(frameCounter <= 5) {
				g_vEndPoint = new CVector3(0, esy -= 0.05f, 0);
				g_vControlPoint2 = new CVector3(c2sx -= 0.1f, c2sy += 0.07f, 0);
				g_vControlPoint1 = new CVector3(c1sx -= 0.3f, c1sy += 0.15f, 0);
				g_vStartPoint = new CVector3(ssx -= 0.35f, ssy += 0.35f, 0);
				
				incFac -= 0.0005f; // Armene er kortere i flyteposisjon, og radiusen m derfor kes mindre
				
				// Flytter blekkspruten litt nedover i flyteposisjonen
				movey -= 0.01f;
				
			// Manipulerer alle kontrollpunktene til  nrme seg samlet armposisjon
			} else {
				g_vEndPoint = new CVector3(0, esy += 0.05f, 0);
				g_vControlPoint2 = new CVector3(c2sx += 0.1f, c2sy -= 0.07f, 0);
				g_vControlPoint1 = new CVector3(c1sx += 0.3f, c1sy -= 0.15f, 0);
				g_vStartPoint = new CVector3(ssx += 0.35f, ssy -= 0.35f, 0);
				
				incFac += 0.0005f; // Armene er litt lengre nr de er samlet, og radiusen m derfor kes mer
				
				// Flytter blekkspruten opp og bakover
				movey += 0.125f;
				movez -= 0.2f;
			}
			
			// Setter opp beziermatrisene p nytt med de nye kontrollpunktene
			makeBezier(g_vStartPoint, g_vControlPoint1, g_vControlPoint2, g_vEndPoint);
			make_d_Bezier(g_vStartPoint, g_vControlPoint1, g_vControlPoint2, g_vEndPoint);
	    }
	}
}