import javax.swing.UIManager;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.lang.*;
import java.util.*;
import java.io.*;

import gl4java.GLContext;
import gl4java.awt.*;
import gl4java.utils.glut.*;
import gl4java.utils.textures.*;

/**
* En klasse som tegner en bezierkurve og perler, bokser eller akser langs kurven.<br>
* Laget for  teste effekten av frenet frames.<br>
* Bruker objekter av klassene CVector3 og Material.
* @author Linda Cecilie Kjeldsen, Grafisk Databehandling 2004, Hi
* @version 0.1
*/

public class Pearls extends JFrame implements ActionListener{
	JPanel contentPane;
	
	// Variabler for musehndtering:
	float rotx = 0.0f;
	float rotz = 0.0f;
	float lastx = 0.0f;
	float lastz = 0.0f;
	float delta_rotx = 1.0f;
	float delta_rotz = 1.0f;
	boolean isMoving = false;
	
	// Variabler for menyen:
	MenuBar menybar;
	Menu meny;
	MenuItem pearls;
	MenuItem boxes;
	MenuItem axis;
	MenuItem directed;
	MenuItem exit;
	
	// Vindus- og musehndterer:
	WindowHandler wHandler;
	MouseHandler mHandler;
	
	// gl4java canvas
	myGLCanvas canvas = null;

	/**
	* Konstruktr for Pearls framen<br>
	* Setter opp menyen, og GLCanvas til  tegne p
	*/
	public Pearls() {
		contentPane = (JPanel) this.getContentPane();
		contentPane.setLayout(new BorderLayout());
		this.setSize(new Dimension(700, 500));
		this.setTitle("Frenet Frames");
		
		// Sett opp menyen
	    menybar = new MenuBar();
	    this.setMenuBar(menybar);
	    meny = new Menu("Frames");
	     
	    menybar.add(meny);
	         
	    pearls = new MenuItem("Perler");
	    meny.add(pearls);
		pearls.addActionListener(this);
		 
		boxes = new MenuItem("Bokser");
	    meny.add(boxes);
		boxes.addActionListener(this);
		
		axis = new MenuItem("Akser");
	    meny.add(axis);
		axis.addActionListener(this);
		
		directed = new MenuItem("Sett retning");
	    meny.add(directed);
		directed.addActionListener(this);
		
	    meny.addSeparator();
	
	    exit = new MenuItem("Exit");
	    meny.add(exit);
	    exit.addActionListener(this);
		
		// Sett opp gl-canvasen
		Dimension d = getSize();
		canvas = new myGLCanvas(d.width, d.height);
		contentPane.add("Center", canvas);
		
		// Sett opp musehndtereren
		mHandler = new MouseHandler();
		canvas.addMouseListener(mHandler);
		canvas.addMouseMotionListener(mHandler);
		
		// Sett opp vindushndtereren
		wHandler=new WindowHandler();
		this.addWindowListener(wHandler);
		
		pack();
		show();
	}

	/**
	* Main metode
	*/
	public static void main(String[] args) {
		new Pearls();
	}
	
	/**
	* Metode for hndtering av brukers handlinger
	* @param e den eventen som skal utfres
	*/ 
	public void actionPerformed(ActionEvent e) {
		if(e.getSource() == pearls) {
			// Tegn perler
			canvas.type = 0;
			canvas.repaint();
		}
		if(e.getSource() == boxes) {
			// Tegn bokser
			canvas.type = 1;
			canvas.repaint();
		}
		if(e.getSource() == axis) {
			// Tegn akser
			canvas.type = 2;
			canvas.repaint();
		}
		if(e.getSource() == directed) {
			// Toggle retning p aksene
			canvas.directed = !canvas.directed;
			canvas.repaint();
		}
		if(e.getSource() == exit) {
			System.exit(0);
		}
	}
	
// --------------------------------------------
	/**
	* Intern klasse for vindushndtering
	* @see WindowListener
	*/
	class WindowHandler implements WindowListener {
		public void windowOpened(WindowEvent parm1) {}
		public void windowClosing(WindowEvent parm1) {
			System.exit( 0 );
		}
		public void windowClosed(WindowEvent parm1) {}
		public void windowIconified(WindowEvent parm1) {}
		public void windowDeiconified(WindowEvent parm1) {}
		public void windowActivated(WindowEvent parm1) {}
		public void windowDeactivated(WindowEvent parm1) {}
	}
	
// --------------------------------------------
	/**
	* Intern klasse for musehndtering
	* @see MouseListener
	* @see MouseMotionListener
	*/
  	class MouseHandler implements MouseListener, MouseMotionListener {
	  	public void mouseEntered(MouseEvent e) {}
	  	public void mouseExited(MouseEvent e) {}
	  	public void mouseMoved(MouseEvent e) {}
	  	public void mouseClicked(MouseEvent e) {}
	  	public void mouseReleased(MouseEvent e) {
		  	isMoving = false;
		  	canvas.repaint();
		}
		public void mousePressed(MouseEvent e) {
			lastx = e.getX();
			lastz = e.getY();
			isMoving = true;
		}
		public void mouseDragged(MouseEvent e) {
			if(!isMoving) return;
			
			int x = e.getX();
			int y = e.getY();
			
			Dimension dim = e.getComponent().getSize();
			int width = dim.width;
			int height = dim.height;
			
			rotz += (90.0f * ((float)(x-lastx)/(float)(width)));
			rotx += (90.0f * ((float)(y-lastz)/(float)(height)));
			
			lastx = x;
			lastz = y;
			
			canvas.repaint();
		}
	}
	
//-------------------------------------------------------
	/**
	* Intern klasse som tar seg av tegningen
	*/
	private class myGLCanvas extends GLCanvas {
		
		GLUTFunc glut = null;
		
		// Variabler for  kontrollere hva som tegnes
		int type = 0;
		boolean directed = false;
		
		Material mat = new Material(gl);
		
		// Variabler for  ta vare p bezierkurvens kontrollpunkter
		int MAX_STEPS = 12; // Antall punkter pr kurve (og antall figurer som tegnes)
		
		float btab[][] = new float[4][MAX_STEPS];	// ligningene for  regne ut bezierfunksjonen
		float d_btab[][] = new float[4][MAX_STEPS];	// ligningene for  regne ut den deriverte
		
		float bez[][] = new float[MAX_STEPS][3]; 	// verdiene til bezierfunksjonen
		float d_bez[][] = new float[MAX_STEPS][3]; 	// verdiene til den deriverte
		
		// Bezierkurvens kontrollpunkter
		CVector3 g_vStartPoint    = new CVector3(-0.5f, 0.5f, 0);
		CVector3 g_vControlPoint1 = new CVector3(-0.3f, -0.4f, 0);
		CVector3 g_vControlPoint2 = new CVector3(0.3f, -0.4f, 0);	
		CVector3 g_vEndPoint	  = new CVector3(0.5f, 0.5f, 0);	
		
		/**
		* Konstruktr for myGLCanvas
		* @param w bredden p komponenten
		* @param h hyden p komponenten
		*/
		public myGLCanvas(int w, int h) {
			super(w, h);
			
			// Sett opp matrisene som holder ligningene for  regne ut kurven og den deriverte
			int tix=0;
			while(tix<MAX_STEPS) {
				float t=(1.0f/(1.0f*MAX_STEPS))*(1.0f*tix);
				btab[0][tix]=(1-t)*(1-t)*(1-t);
				btab[1][tix]=3.0f*t*(1-t)*(1-t);
				btab[2][tix]=3.0f*t*t*(1-t);
				btab[3][tix]=t*t*t;
				tix++;
			}
			tix=0;
			while(tix<MAX_STEPS) {
				float t=(1.0f/(1.0f*MAX_STEPS))*(1.0f*tix);
				d_btab[0][tix]=-3.0f*t*t+6.0f*t-3.0f;
				d_btab[1][tix]=9.0f*t*t-12.0f*t+3.0f;
				d_btab[2][tix]=-9.0f*t*t+6.0f*t;
				d_btab[3][tix]=3.0f*t*t;
				tix++;
			}
			
			makeBezier(g_vStartPoint, g_vControlPoint1, g_vControlPoint2, g_vEndPoint);
			make_d_Bezier(g_vStartPoint, g_vControlPoint1, g_vControlPoint2, g_vEndPoint);
		}
		
		/**
	     * Initialiserer canvasen fr GLContext er opprettet.
	     * <p>
	     * Blir kalt like FR GLContext blir opprettet.
	     * <p>
	     * Definerer et fornuftig sett med 3D options til denne komponenten.  
	     */
		public void preInit() {
			doubleBuffer = true;
			stereoView = false;
			accumSize = 8;
		}
		
		/**
	     * Initialiserer canvasen etter at GLContext er opprettet.
	     * <p>
	     * Blir kalt like ETTER GLContext er opprettet. Setter opp OpenGLs tilstandsmatrise.
	     * Setter opp lyset, glatter modellen, aktiviserer dybdetesting, og setter bakgrunnsfarge.
	     */
		public void init() {
			glut = new GLUTFuncLightImpl(gl, glu);
			reshape(getSize().width, getSize().height);
			
			// Setter opp lyset
			float ambient[] = {0.4f,0.4f,0.4f,1.0f };
			float diffuse[] = {1.0f,1.0f,1.0f,1.0f };
			float position[] = {1.0f,1.0f,1.0f,0.0f };
			float lmodel_ambient[] = {0.4f,0.4f,0.4f,1.0f };
			gl.glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
			gl.glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
			gl.glLightfv(GL_LIGHT0, GL_POSITION, position);
			
			gl.glEnable(GL_LIGHT0);
			gl.glEnable(GL_LIGHTING);
			
			gl.glShadeModel(GL_SMOOTH);
			
			// setter bakgrunnen til hvit
			gl.glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
		}
		
		/**
		* Forsikrer at GL er korrekt initialisert.
		* Kaller showScene() hvis GL4Java er initialisert, og gjr
		* GLContexten current for denne trden
		* 
		* Klarerer de ndvendige bufferene, setter opp transformasjonsmatrisen for viewet,
		* sjekker hva som skal tegnes, og tegner kurven og figurene.
		*/
		public void display() {
			if (glj.gljMakeCurrent() == false) return;

			gl.glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clear The Screen And The Depth Buffer
			gl.glLoadIdentity();									// Reset The matrix

			// Setter opp view-matrisen				      		
			glu.gluLookAt(0.0f, 0.0f, 4.0f, // yets posisjon 
						0.0f, 0.0f, 0.0f, 	// Hva ser vi p
						0.0f, 1.0f, 0.0f);	// Hva er opp
			
			gl.glRotatef(rotx, 1.0f, 0.0f, 0.0f);
			gl.glRotatef(rotz, 0.0f, 0.0f, 1.0f);			
			
			// Tegn bezierkurven
			gl.glDisable(GL_LIGHTING);	// Slr av lyset for  se kurven klart
			gl.glColor3f(0,0,0);		// Setter fargen p kurven til svart
			gl.glLineWidth(2);			// Vil ha en litt bredere linje
			drawBezier();				// Kaller metoden som tegner kurven "for hnd"
			gl.glEnable(GL_LIGHTING);	// Slr p lyset igjen
			
			// Tegn figurene
			int ix=0;
			while(ix < MAX_STEPS) {
				// Sjekker om aksene skal vre rettet etter kurven eller ikke
				if(directed) {
					// Sett opp vektorer for  sette retning p aksene 
					// For ytterlige kommentarer, se Octopus.java
					CVector3 T = new CVector3(d_bez[ix][0],d_bez[ix][1],d_bez[ix][2]);
					T.normalize();
					
					CVector3 B = new CVector3(d_bez[ix][0],d_bez[ix][1],d_bez[ix][2]);
					B.cross(0.0f,0.0f,1.0f);
					B.normalize();
					
					CVector3 N = new CVector3(B.x,B.y,B.z);
					N.cross(T); 
			
					CVector3 C = new CVector3(bez[ix][0],bez[ix][1],bez[ix][2]);
			
					float M[] = {
						N.x,N.y,N.z,0,
						B.x,B.y,B.z,0,
						T.x,T.y,T.z,0,
						C.x,C.y,C.z,1
					};
					
					gl.glPushMatrix();
					gl.glMultMatrixf(M);
				// Flytter origo til det aktuelle punktet p kurven, uten  rotere aksene
				} else {
					gl.glPushMatrix();
					gl.glTranslatef(bez[ix][0],bez[ix][1],bez[ix][2]);
				}
				
				// Tegn perler:
				if(type == 0) {
					mat.white();
					long glSphere = glu.gluNewQuadric();
					glu.gluSphere(glSphere,0.05f,20,20);
					glu.gluDeleteQuadric(glSphere);
				// Bokser:
				} else if(type == 1){
					mat.purple();
					gl.glTranslatef(-0.05f,-0.05f,-0.05f);
					drawBox();
				// Akser:
				} else {
					gl.glDisable(GL_LIGHTING);
					gl.glColor3f(0,0,0);
					gl.glLineWidth(1);
					
					gl.glBegin(GL_LINES);
						gl.glVertex3f(0.0f,0.0f,0.0f);
						gl.glVertex3f(0.0f,0.0f,0.2f);
					gl.glEnd();
					gl.glBegin(GL_LINES);
						gl.glVertex3f(0.0f,0.0f,0.0f);
						gl.glVertex3f(0.0f,0.2f,0.0f);
					gl.glEnd();
					gl.glBegin(GL_LINES);
						gl.glVertex3f(0.0f,0.0f,0.0f);
						gl.glVertex3f(0.2f,0.0f,0.0f);
					gl.glEnd();
										
					gl.glEnable(GL_LIGHTING);
				}
				
				gl.glPopMatrix();
				ix++;
			}
			// Swap bufferne og frigjr contexten
			glj.gljSwap();
			glj.gljFree();
		}
		
		/**
		* Hjelpefunksjon som tegner bezierfunksjonen "for hnd", 
		* alle punkter er kalkulert p forhnd
		*/ 
		private void drawBezier() {
			gl.glBegin(GL_LINE_STRIP);
			int ix = 0;
			while(ix < MAX_STEPS)
			{
				gl.glVertex3f(bez[ix][0],bez[ix][1],bez[ix][2]);
				ix++;
			}
			gl.glEnd();
		}
		
		/**
		* Tegner en boks med seks like store flater
		*/
		public void drawBox() {
			gl.glBegin(GL_POLYGON);
				gl.glNormal3f(-1.0f, 0.0f, 0.0f);
				gl.glVertex3f(0.0f, 0.0f, 0.0f);
				gl.glVertex3f(0.0f, 0.0f, 0.1f);
				gl.glVertex3f(0.1f, 0.0f, 0.1f);
				gl.glVertex3f(0.1f, 0.0f, 0.0f);
			gl.glEnd();
			gl.glBegin(GL_POLYGON);
				gl.glNormal3f(0.0f, 0.0f, -1.0f);
				gl.glVertex3f(0.0f, 0.0f, 0.0f);
				gl.glVertex3f(0.1f, 0.0f, 0.0f);
				gl.glVertex3f(0.1f, 0.1f, 0.0f);
				gl.glVertex3f(0.0f, 0.1f, 0.0f);
			gl.glEnd();
			gl.glBegin(GL_POLYGON);
				gl.glNormal3f(1.0f, 0.0f, 0.0f);
				gl.glVertex3f(0.1f, 0.1f, 0.0f);
				gl.glVertex3f(0.1f, 0.1f, 0.1f);
				gl.glVertex3f(0.0f, 0.1f, 0.1f);
				gl.glVertex3f(0.0f, 0.1f, 0.0f);
			gl.glEnd();
			gl.glBegin(GL_POLYGON);
				gl.glNormal3f(0.0f, 0.0f, 1.0f);
				gl.glVertex3f(0.1f, 0.1f, 0.1f);
				gl.glVertex3f(0.1f, 0.0f, 0.1f);
				gl.glVertex3f(0.0f, 0.0f, 0.1f);
				gl.glVertex3f(0.0f, 0.1f, 0.1f);
			gl.glEnd();
			gl.glBegin(GL_POLYGON);
				gl.glNormal3f(0.0f, 1.0f, 0.0f);
				gl.glVertex3f(0.0f, 0.0f, 0.0f);
				gl.glVertex3f(0.0f, 0.1f, 0.0f);
				gl.glVertex3f(0.0f, 0.1f, 0.1f);
				gl.glVertex3f(0.0f, 0.0f, 0.1f);
			gl.glEnd();
			gl.glBegin(GL_POLYGON);
				gl.glNormal3f(0.0f, -1.0f, 0.0f);
				gl.glVertex3f(0.1f, 0.0f, 0.0f);
				gl.glVertex3f(0.1f, 0.0f, 0.1f);
				gl.glVertex3f(0.1f, 0.1f, 0.1f);
				gl.glVertex3f(0.1f, 0.1f, 0.0f);
			gl.glEnd();
		}
		
		/**
		* Setter opp matrisen som holder punktene til bezierfunksjonen
		*/
		private void makeBezier(CVector3 p0, CVector3 p1, CVector3 p2, CVector3 p3) {
			int ix = 0;
			while(ix < MAX_STEPS)
			{
				bez[ix][0]=p0.x*btab[0][ix]+p1.x*btab[1][ix]+p2.x*btab[2][ix]+p3.x*btab[3][ix];
				bez[ix][1]=p0.y*btab[0][ix]+p1.y*btab[1][ix]+p2.y*btab[2][ix]+p3.y*btab[3][ix];
				bez[ix][2]=p0.z*btab[0][ix]+p1.z*btab[1][ix]+p2.z*btab[2][ix]+p3.z*btab[3][ix];
				
				ix++;
			}
		}
		
		/**
		* Setter opp matrisen som holder punktene til den deriverte
		*/
		private void make_d_Bezier(CVector3 p0, CVector3 p1, CVector3 p2, CVector3 p3) {
			int ix = 0;
			while(ix < MAX_STEPS)
			{
				d_bez[ix][0]=p0.x*d_btab[0][ix]+p1.x*d_btab[1][ix]+p2.x*d_btab[2][ix]+p3.x*d_btab[3][ix];
				d_bez[ix][1]=p0.y*d_btab[0][ix]+p1.y*d_btab[1][ix]+p2.y*d_btab[2][ix]+p3.y*d_btab[3][ix];
				d_bez[ix][2]=p0.z*d_btab[0][ix]+p1.z*d_btab[1][ix]+p2.z*d_btab[2][ix]+p3.z*d_btab[3][ix];
				
				ix++;
			}
		}
		
		/**
		* Kalles nr strrelsen p canvasen endres
		* @param w bredden p komponenten
		* @param h hyden p komponenten
		*/
		public void reshape(int w, int h) {
			gl.glViewport(0, 0, w, h);
			gl.glMatrixMode(GL_PROJECTION);
			gl.glLoadIdentity();
			glu.gluPerspective(45.0f,(float)w/(float)h, 0.5 ,150.0f);
			gl.glMatrixMode(GL_MODELVIEW);
			gl.glLoadIdentity();
		}
	}
}